# 1 "wikicreole.mll"
 
(* Ocsimore
 * Copyright (C) 2008
 * Laboratoire PPS - Universit Paris Diderot - CNRS
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *)
(**
   Parser for Wikicreole
   @author Jrme Vouillon
*)

type ('flow, 'inline, 'a_content) builder =
  { chars : string -> 'a_content;
    strong_elem : 'inline list -> 'a_content;
    em_elem : 'inline list -> 'a_content;
    br_elem : unit -> 'a_content;
    img_elem : string -> string -> 'a_content;
    tt_elem : 'inline list -> 'a_content;
    a_elem : string -> 'a_content list -> 'inline;
    youtube_elem : string -> 'a_content list -> 'inline;
    p_elem : 'inline list -> 'flow;
    pre_elem : string list -> 'flow;
    h1_elem : 'inline list -> 'flow;
    h2_elem : 'inline list -> 'flow;
    h3_elem : 'inline list -> 'flow;
    h4_elem : 'inline list -> 'flow;
    h5_elem : 'inline list -> 'flow;
    h6_elem : 'inline list -> 'flow;
    ul_elem : ('inline list * 'flow option) list -> 'flow;
    ol_elem : ('inline list * 'flow option) list -> 'flow;
    hr_elem : unit -> 'flow;
    table_elem : (bool * 'inline list) list list -> 'flow;
    inline : 'a_content -> 'inline }

type style = Bold | Italic

type list_kind = Unordered | Ordered

type ('inline, 'flow) stack =
    Style of style * 'inline list * ('inline, 'flow) stack
  | Link of string * ('inline, 'flow) stack
      (* Not that we do not save anything in the case of links, as
         links cannot be nested *)
  | Paragraph
  | Heading of int
  | List_item of ('inline, 'flow) stack
  | List of
      list_kind * ('inline list * 'flow option) list * ('inline, 'flow) stack
  | Table of (bool * 'inline list) list list
  | Row of (bool * 'inline list) list * ('inline, 'flow) stack
  | Entry of bool * ('inline, 'flow) stack

type ('flow, 'inline, 'a_content) ctx =
  { build : ('flow, 'inline, 'a_content) builder;
    mutable italic : bool;
    mutable bold : bool;
    mutable heading : bool;
    mutable link : bool;
    mutable list_level : int;
    mutable inline_mix : 'inline list;
    mutable link_content : 'a_content list;
    mutable pre_content : string list;
    mutable list : ('inline list * 'flow option) list;
    mutable flow : 'flow list;
    mutable stack : ('inline, 'flow) stack }

let count c s =
  let n = ref 0 in
  for i = 0 to String.length s - 1 do if s.[i] = c then incr n done;
  !n

let push c v =
  match c.stack with
    Link _ -> c.link_content <- v :: c.link_content
  | _      -> c.inline_mix <- c.build.inline v :: c.inline_mix

let push_string c s = push c (c.build.chars s)

let push_chars c lexbuf = push_string c (Lexing.lexeme lexbuf)

let get_style c style =
  match style with Bold -> c.bold | Italic -> c.italic

let set_style c style v =
  match style with Bold -> c.bold <- v | Italic -> c.italic <- v

let pop_style c style inline stack =
  let elt =
    match style with
      Bold   -> c.build.strong_elem
    | Italic -> c.build.em_elem
  in
  let inline' = c.inline_mix in
  c.stack <- stack;
  c.inline_mix <- inline;
  push c (elt (List.rev inline'));
  set_style c style false

let style_change c style =
  if get_style c style then begin
    match c.stack with
      Style (s, inline, stack) when s = style ->
        pop_style c style inline stack
    | _ ->
        match style with
          Italic -> push_string c "//"
        | Bold   -> push_string c "**"
  end else begin
    c.stack <- Style (style, c.inline_mix, c.stack);
    c.inline_mix <- [];
    set_style c style true
  end

let pop_link c addr stack =
  c.stack <- stack;
  c.inline_mix <-
    c.build.a_elem addr (List.rev c.link_content) :: c.inline_mix;
  c.link_content <- [];
  c.link <- false

let close_entry c =
  match c.stack with
    Entry (heading, Row (entries, stack)) ->
      c.stack <- Row ((heading, List.rev c.inline_mix) :: entries, stack);
      c.inline_mix <- [];
      true
  | Row _ ->
      true
  | Table _ ->
      c.stack <- Row ([(false, List.rev c.inline_mix)], c.stack);
      c.inline_mix <- [];
      true
  | _ ->
      false

let close_row c =
  close_entry c &&
  match c.stack with
    Row (entries, Table rows) ->
      c.stack <- Table (List.rev entries :: rows);
      true
  | Table _ ->
      true
  | _ ->
      assert false

let rec end_paragraph c lev =
  match c.stack with
    Style (style, inline, stack) ->
      pop_style c style inline stack;
      end_paragraph c lev
  | Link (addr, stack) ->
      pop_link c addr stack;
      end_paragraph c lev
  | Paragraph ->
      if c.inline_mix <> [] then begin
        c.flow <- c.build.p_elem (List.rev c.inline_mix) :: c.flow;
        c.inline_mix <- []
      end;
      c.stack <- Paragraph
  | Heading l ->
      let f =
        match l with
          | 1 -> c.build.h1_elem
          | 2 -> c.build.h2_elem
          | 3 -> c.build.h3_elem
          | 4 -> c.build.h4_elem
          | 5 -> c.build.h5_elem
          | _ -> c.build.h6_elem
      in
      c.flow <- f (List.rev c.inline_mix) :: c.flow;
      c.inline_mix <- [];
      c.heading <- false;
      c.stack <- Paragraph
  | List_item stack ->
      c.list <- (List.rev c.inline_mix, None) :: c.list;
      c.stack <- stack;
      c.inline_mix <- [];
      end_paragraph c lev
  | List (kind, lst, stack) ->
      if lev < c.list_level then begin
        c.list_level <- c.list_level - 1;
        let elt =
          match kind with
            Unordered -> c.build.ul_elem
          | Ordered   -> c.build.ol_elem
        in
        let cur_lst = elt (List.rev c.list) in
        if c.list_level = 0 then
          c.flow <- cur_lst :: c.flow
        else begin
          match lst with
            (l, None) :: rem -> c.list <- (l, Some cur_lst) :: rem;
          | _                -> assert false
        end;
        c.stack <- stack;
        end_paragraph c lev
      end
  | Entry _ ->
      ignore (close_row c);
      end_paragraph c lev
  | Row _ ->
      assert false
  | Table rows ->
      c.flow <- c.build.table_elem (List.rev rows) :: c.flow;
      c.stack <- Paragraph

let rec correct_kind_rec stack kind n =
  match stack with
    List_item stack ->
      correct_kind_rec stack kind n
  | List (k, lst, stack) ->
      if n = 0 then k = kind else
      correct_kind_rec stack kind (n - 1)
  | Style (_, _, stack) ->
      correct_kind_rec stack kind n
  | Link _ | Heading _ | Paragraph | Entry _ | Row _ | Table _ ->
      assert false

let correct_kind c kind lev =
  lev = c.list_level + 1
    ||
  (lev <= c.list_level &&
   correct_kind_rec c.stack kind (c.list_level - lev))

let start_list_item c kind lev =
  let correct = correct_kind c kind lev in
  if lev = 1 || correct then begin
    (* If we have an item of a different kind at level 1, we close the
       previous list and start a new one of the right kind *)
    end_paragraph c (if correct then lev else 0);
    if lev = c.list_level then begin
      c.stack <- List_item c.stack
    end else (* if lev = c.list_level + 1 then *) begin
      c.list_level <- lev;
      c.stack <- List_item (List (kind, c.list, c.stack));
      c.list <- []
    end;
    true
  end else
    false

let start_table_row c heading =
  if not (close_row c) then begin
    end_paragraph c 0;
    c.stack <- Table []
  end;
  c.stack <- Entry (heading, Row ([], c.stack))


# 266 "wikicreole.ml"
let __ocaml_lex_tables = {
  Lexing.lex_base = 
   "\000\000\001\000\002\000\001\000\001\000\001\000\002\000\005\000\
    \001\000\255\255\003\000\004\000\006\000\007\000\254\255\003\000\
    \004\000\006\000\251\255\002\000\003\000\007\000\250\255\008\000\
    \248\255\011\000\238\255\047\000\020\000\046\000\070\000\085\000\
    \108\000\155\000\193\000\208\000\008\001\025\001\077\001\081\001\
    \012\000\255\255\254\255\253\255\252\255\013\000\148\001\064\000\
    \066\000\074\000\249\255\120\000\251\255\152\001\204\001\219\001\
    \001\002\053\002\069\002\121\002\159\002\174\002\031\000\227\002\
    \245\002\025\003\042\003\096\000\250\255\248\255\089\003\095\003\
    \142\003\215\003\014\004\058\004\100\004\105\004\128\004\246\255\
    \106\000\097\000\215\000\135\000\171\000\245\255\182\000\210\000\
    \011\000\243\255\240\255\242\255\015\000\143\000\112\001\016\000\
    \253\255\223\000\254\255\101\001\143\001\123\001\137\001\228\000\
    \255\255\017\000\154\001\004\001\018\000";
  Lexing.lex_backtrk = 
   "\008\000\006\000\255\255\255\255\003\000\002\000\001\000\255\255\
    \000\000\255\255\001\000\001\000\001\000\001\000\255\255\255\255\
    \255\255\255\255\255\255\004\000\255\255\255\255\255\255\005\000\
    \255\255\255\255\255\255\016\000\014\000\016\000\016\000\016\000\
    \016\000\016\000\016\000\016\000\016\000\016\000\016\000\016\000\
    \000\000\255\255\255\255\255\255\255\255\003\000\016\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\016\000\016\000\016\000\
    \016\000\016\000\016\000\016\000\016\000\016\000\255\255\016\000\
    \016\000\016\000\005\000\255\255\255\255\255\255\016\000\016\000\
    \016\000\016\000\008\000\008\000\255\255\016\000\016\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\011\000\
    \011\000\255\255\255\255\255\255\013\000\255\255\255\255\002\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\000\000\255\255\255\255\001\000";
  Lexing.lex_default = 
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\000\000\255\255\255\255\255\255\255\255\000\000\255\255\
    \255\255\255\255\000\000\255\255\255\255\255\255\000\000\255\255\
    \000\000\027\000\000\000\027\000\255\255\089\000\027\000\027\000\
    \027\000\027\000\027\000\027\000\027\000\027\000\027\000\027\000\
    \255\255\000\000\000\000\000\000\000\000\255\255\027\000\048\000\
    \048\000\048\000\000\000\048\000\000\000\027\000\027\000\027\000\
    \027\000\027\000\027\000\027\000\027\000\063\000\062\000\063\000\
    \063\000\063\000\027\000\062\000\000\000\000\000\027\000\027\000\
    \027\000\075\000\074\000\075\000\074\000\027\000\027\000\000\000\
    \082\000\081\000\082\000\083\000\083\000\000\000\081\000\081\000\
    \255\255\000\000\000\000\000\000\255\255\255\255\097\000\255\255\
    \000\000\097\000\000\000\097\000\097\000\097\000\097\000\097\000\
    \000\000\255\255\097\000\097\000\255\255";
  Lexing.lex_trans = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\007\000\009\000\009\000\018\000\008\000\007\000\017\000\
    \018\000\022\000\022\000\019\000\023\000\041\000\041\000\044\000\
    \040\000\091\000\096\000\104\000\098\000\093\000\091\000\000\000\
    \007\000\092\000\000\000\004\000\004\000\007\000\017\000\000\000\
    \004\000\255\255\005\000\005\000\255\255\003\000\015\000\005\000\
    \016\000\017\000\003\000\000\000\093\000\039\000\000\000\255\255\
    \255\255\255\255\038\000\255\255\255\255\006\000\024\000\010\000\
    \011\000\012\000\006\000\013\000\014\000\000\000\000\000\035\000\
    \037\000\000\000\255\255\000\000\255\255\255\255\255\255\255\255\
    \255\255\090\000\000\000\255\255\255\255\000\000\000\000\255\255\
    \000\000\255\255\000\000\000\000\000\000\067\000\255\255\255\255\
    \000\000\000\000\255\255\000\000\000\000\000\000\036\000\031\000\
    \034\000\000\000\255\255\255\255\255\255\255\255\255\255\000\000\
    \255\255\032\000\000\000\033\000\255\255\255\255\255\255\255\255\
    \000\000\255\255\000\000\002\000\001\000\020\000\021\000\255\255\
    \002\000\001\000\255\255\255\255\255\255\255\255\030\000\028\000\
    \088\000\029\000\255\255\255\255\255\255\000\000\000\000\000\000\
    \000\000\255\255\255\255\000\000\255\255\255\255\255\255\255\255\
    \093\000\091\000\000\000\255\255\092\000\049\000\068\000\051\000\
    \000\000\255\255\255\255\255\255\000\000\255\255\000\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\093\000\
    \255\255\079\000\255\255\000\000\000\000\255\255\000\000\000\000\
    \255\255\000\000\000\000\255\255\255\255\255\255\050\000\000\000\
    \255\255\080\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\000\000\000\000\255\255\000\000\
    \255\255\255\255\255\255\255\255\255\255\052\000\000\000\000\000\
    \255\255\000\000\255\255\000\000\255\255\255\255\086\000\255\255\
    \077\000\255\255\000\000\000\000\255\255\081\000\255\255\255\255\
    \255\255\096\000\255\255\255\255\095\000\000\000\104\000\000\000\
    \255\255\105\000\000\000\000\000\255\255\000\000\255\255\255\255\
    \255\255\000\000\255\255\000\000\000\000\000\000\255\255\255\255\
    \000\000\255\255\000\000\255\255\084\000\000\000\018\000\022\000\
    \000\000\000\000\000\000\026\000\053\000\255\255\098\000\070\000\
    \000\000\108\000\255\255\000\000\091\000\255\255\255\255\255\255\
    \000\000\255\255\000\000\000\000\255\255\255\255\069\000\255\255\
    \000\000\000\000\046\000\044\000\000\000\000\000\045\000\255\255\
    \085\000\255\255\000\000\255\255\255\255\255\255\255\255\255\255\
    \000\000\000\000\255\255\087\000\000\000\000\000\255\255\255\255\
    \255\255\046\000\000\000\000\000\255\255\255\255\000\000\255\255\
    \255\255\000\000\255\255\255\255\000\000\255\255\255\255\000\000\
    \255\255\000\000\255\255\255\255\255\255\000\000\255\255\088\000\
    \000\000\000\000\000\000\083\000\000\000\255\255\037\000\255\255\
    \000\000\000\000\255\255\255\255\000\000\000\000\255\255\000\000\
    \255\255\255\255\000\000\047\000\255\255\255\255\000\000\000\000\
    \000\000\000\000\255\255\000\000\255\255\000\000\255\255\096\000\
    \255\255\000\000\095\000\000\000\255\255\255\255\255\255\255\255\
    \255\255\100\000\096\000\042\000\043\000\095\000\000\000\255\255\
    \255\255\255\255\000\000\255\255\255\255\096\000\255\255\255\255\
    \095\000\000\000\255\255\000\000\000\000\000\000\255\255\091\000\
    \100\000\000\000\000\000\096\000\255\255\255\255\095\000\255\255\
    \100\000\096\000\000\000\255\255\095\000\046\000\044\000\000\000\
    \000\000\045\000\255\255\000\000\096\000\255\255\000\000\095\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\100\000\
    \000\000\000\000\000\000\255\255\046\000\255\255\255\255\255\255\
    \000\000\255\255\000\000\000\000\000\000\000\000\255\255\000\000\
    \000\000\255\255\255\255\255\255\000\000\000\000\000\000\255\255\
    \255\255\255\255\000\000\255\255\255\255\255\255\000\000\255\255\
    \255\255\255\255\255\255\000\000\000\000\255\255\255\255\255\255\
    \000\000\255\255\000\000\000\000\000\000\000\000\000\000\096\000\
    \000\000\000\000\106\000\000\000\104\000\255\255\000\000\000\000\
    \255\255\000\000\000\000\000\000\000\000\099\000\000\000\255\255\
    \255\255\255\255\000\000\255\255\255\255\255\255\255\255\000\000\
    \102\000\000\000\255\255\255\255\255\255\000\000\255\255\000\000\
    \255\255\000\000\000\000\000\000\098\000\255\255\103\000\000\000\
    \255\255\255\255\255\255\255\255\101\000\000\000\255\255\255\255\
    \255\255\054\000\255\255\255\255\255\255\000\000\255\255\107\000\
    \255\255\044\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\255\255\
    \255\255\255\255\000\000\255\255\000\000\000\000\000\000\000\000\
    \255\255\000\000\255\255\000\000\255\255\000\000\255\255\255\255\
    \255\255\000\000\000\000\055\000\000\000\000\000\255\255\255\255\
    \000\000\255\255\255\255\255\255\000\000\000\000\000\000\255\255\
    \255\255\000\000\255\255\000\000\000\000\255\255\000\000\255\255\
    \056\000\255\255\255\255\000\000\000\000\000\000\255\255\255\255\
    \000\000\255\255\000\000\000\000\255\255\255\255\255\255\255\255\
    \000\000\000\000\000\000\000\000\255\255\096\000\000\000\255\255\
    \000\000\255\255\000\000\000\000\000\000\000\000\000\000\255\255\
    \098\000\000\000\255\255\000\000\255\255\057\000\000\000\000\000\
    \000\000\000\000\000\000\096\000\255\255\255\255\000\000\255\255\
    \000\000\000\000\255\255\255\255\000\000\000\000\255\255\000\000\
    \000\000\096\000\000\000\000\000\000\000\000\000\000\000\096\000\
    \255\255\255\255\255\255\000\000\044\000\000\000\000\000\000\000\
    \255\255\000\000\096\000\255\255\000\000\255\255\000\000\000\000\
    \255\255\255\255\255\255\255\255\000\000\000\000\000\000\059\000\
    \255\255\255\255\058\000\255\255\255\255\255\255\000\000\000\000\
    \255\255\255\255\000\000\255\255\000\000\000\000\255\255\000\000\
    \255\255\000\000\000\000\255\255\000\000\000\000\000\000\061\000\
    \255\255\255\255\000\000\255\255\000\000\000\000\000\000\000\000\
    \000\000\255\255\000\000\000\000\255\255\000\000\255\255\000\000\
    \000\000\000\000\000\000\000\000\255\255\255\255\255\255\000\000\
    \062\000\000\000\000\000\255\255\255\255\062\000\060\000\255\255\
    \000\000\255\255\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\062\000\064\000\255\255\000\000\000\000\
    \255\255\000\000\000\000\000\000\255\255\255\255\000\000\255\255\
    \000\000\000\000\255\255\255\255\255\255\000\000\000\000\255\255\
    \000\000\255\255\255\255\000\000\000\000\255\255\000\000\255\255\
    \000\000\062\000\062\000\062\000\000\000\062\000\000\000\000\000\
    \000\000\000\000\062\000\000\000\062\000\000\000\062\000\000\000\
    \000\000\000\000\255\255\255\255\000\000\255\255\000\000\062\000\
    \062\000\065\000\000\000\255\255\062\000\000\000\255\255\000\000\
    \000\000\062\000\062\000\000\000\062\000\000\000\000\000\000\000\
    \000\000\000\000\062\000\027\000\255\255\255\255\000\000\255\255\
    \000\000\000\000\000\000\000\000\000\000\000\000\062\000\062\000\
    \062\000\000\000\000\000\062\000\000\000\255\255\000\000\000\000\
    \062\000\062\000\000\000\062\000\000\000\000\000\000\000\000\000\
    \062\000\062\000\062\000\000\000\255\255\000\000\062\000\066\000\
    \000\000\255\255\000\000\062\000\000\000\062\000\062\000\062\000\
    \000\000\062\000\000\000\255\255\000\000\000\000\255\255\255\255\
    \000\000\255\255\000\000\000\000\255\255\000\000\000\000\000\000\
    \062\000\062\000\000\000\062\000\062\000\062\000\062\000\000\000\
    \000\000\255\255\000\000\000\000\000\000\000\000\000\000\062\000\
    \000\000\062\000\000\000\255\255\000\000\255\255\255\255\255\255\
    \255\255\255\255\000\000\000\000\000\000\000\000\255\255\000\000\
    \255\255\000\000\255\255\000\000\062\000\062\000\255\255\062\000\
    \255\255\000\000\000\000\255\255\255\255\000\000\000\000\255\255\
    \000\000\000\000\000\000\000\000\000\000\255\255\255\255\000\000\
    \255\255\000\000\000\000\000\000\000\000\000\000\255\255\000\000\
    \000\000\000\000\000\000\000\000\255\255\255\255\255\255\000\000\
    \255\255\000\000\255\255\255\255\255\255\255\255\000\000\255\255\
    \000\000\255\255\000\000\000\000\000\000\255\255\000\000\255\255\
    \073\000\000\000\000\000\255\255\000\000\071\000\000\000\072\000\
    \000\000\000\000\000\000\000\000\255\255\255\255\000\000\255\255\
    \000\000\000\000\255\255\255\255\000\000\255\255\000\000\000\000\
    \027\000\255\255\000\000\255\255\255\255\000\000\000\000\000\000\
    \000\000\255\255\255\255\255\255\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\255\255\255\255\255\255\027\000\
    \073\000\073\000\000\000\000\000\000\000\000\000\073\000\000\000\
    \000\000\074\000\000\000\073\000\000\000\073\000\074\000\000\000\
    \000\000\255\255\255\255\000\000\255\255\000\000\000\000\000\000\
    \000\000\073\000\073\000\000\000\074\000\000\000\073\000\255\255\
    \255\255\255\255\000\000\255\255\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\255\255\000\000\000\000\000\000\255\255\076\000\
    \076\000\000\000\074\000\074\000\074\000\076\000\000\000\000\000\
    \000\000\000\000\076\000\000\000\076\000\074\000\000\000\074\000\
    \000\000\000\000\000\000\027\000\255\255\000\000\000\000\255\255\
    \076\000\076\000\000\000\000\000\000\000\076\000\000\000\000\000\
    \000\000\000\000\074\000\074\000\000\000\074\000\000\000\000\000\
    \000\000\255\255\027\000\073\000\073\000\000\000\000\000\255\255\
    \000\000\073\000\000\000\000\000\074\000\000\000\073\000\000\000\
    \073\000\074\000\000\000\000\000\000\000\255\255\255\255\000\000\
    \000\000\255\255\000\000\255\255\073\000\073\000\255\255\074\000\
    \000\000\073\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\255\255\076\000\076\000\000\000\
    \000\000\000\000\255\255\076\000\000\000\255\255\255\255\000\000\
    \076\000\000\000\076\000\255\255\000\000\074\000\074\000\074\000\
    \255\255\000\000\000\000\000\000\000\000\000\000\076\000\076\000\
    \074\000\000\000\074\000\076\000\000\000\000\000\255\255\000\000\
    \000\000\000\000\255\255\000\000\000\000\000\000\000\000\255\255\
    \000\000\000\000\000\000\000\000\000\000\074\000\074\000\000\000\
    \074\000\000\000\073\000\000\000\000\000\255\255\000\000\000\000\
    \000\000\000\000\000\000\000\000\255\255\255\255\255\255\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\255\255\
    \000\000\255\255\000\000\000\000\000\000\000\000\000\000\255\255\
    \000\000\078\000\000\000\255\255\255\255\255\255\000\000\000\000\
    \000\000\000\000\000\000\000\000\255\255\255\255\255\255\255\255\
    \255\255\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\255\255\255\255\000\000\255\255\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\255\255\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\255\255\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\255\255\000\000\000\000\000\000\
    \000\000\255\255\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \255\255";
  Lexing.lex_check = 
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\000\000\000\000\008\000\019\000\000\000\007\000\017\000\
    \017\000\021\000\023\000\017\000\021\000\025\000\040\000\045\000\
    \025\000\092\000\095\000\105\000\108\000\028\000\028\000\255\255\
    \000\000\028\000\255\255\000\000\004\000\007\000\017\000\255\255\
    \007\000\062\000\000\000\005\000\062\000\000\000\003\000\007\000\
    \015\000\016\000\007\000\255\255\028\000\025\000\255\255\029\000\
    \029\000\027\000\025\000\029\000\027\000\000\000\001\000\006\000\
    \010\000\011\000\007\000\012\000\013\000\255\255\255\255\025\000\
    \025\000\255\255\047\000\255\255\048\000\047\000\029\000\048\000\
    \030\000\028\000\255\255\030\000\049\000\255\255\255\255\049\000\
    \255\255\027\000\255\255\255\255\255\255\062\000\027\000\031\000\
    \255\255\255\255\031\000\255\255\255\255\255\255\025\000\025\000\
    \025\000\255\255\067\000\081\000\027\000\067\000\081\000\255\255\
    \030\000\025\000\255\255\025\000\080\000\030\000\032\000\080\000\
    \255\255\032\000\255\255\000\000\000\000\002\000\020\000\031\000\
    \007\000\007\000\051\000\030\000\031\000\051\000\025\000\025\000\
    \088\000\025\000\027\000\027\000\027\000\255\255\255\255\255\255\
    \255\255\083\000\031\000\255\255\083\000\027\000\032\000\027\000\
    \093\000\093\000\255\255\032\000\093\000\047\000\067\000\048\000\
    \255\255\030\000\030\000\030\000\255\255\033\000\255\255\049\000\
    \033\000\032\000\027\000\027\000\030\000\027\000\030\000\093\000\
    \031\000\031\000\031\000\255\255\255\255\084\000\255\255\255\255\
    \084\000\255\255\255\255\031\000\047\000\031\000\048\000\255\255\
    \086\000\030\000\030\000\086\000\030\000\033\000\049\000\032\000\
    \032\000\032\000\033\000\034\000\255\255\255\255\034\000\255\255\
    \031\000\031\000\032\000\031\000\032\000\051\000\255\255\255\255\
    \033\000\255\255\035\000\255\255\087\000\035\000\081\000\087\000\
    \032\000\082\000\255\255\255\255\082\000\080\000\080\000\032\000\
    \032\000\097\000\032\000\034\000\097\000\255\255\103\000\255\255\
    \034\000\103\000\255\255\255\255\051\000\255\255\033\000\033\000\
    \033\000\255\255\035\000\255\255\255\255\255\255\034\000\035\000\
    \255\255\033\000\255\255\033\000\083\000\255\255\017\000\021\000\
    \255\255\255\255\255\255\025\000\035\000\035\000\107\000\033\000\
    \255\255\107\000\036\000\255\255\028\000\036\000\033\000\033\000\
    \255\255\033\000\255\255\255\255\034\000\034\000\034\000\062\000\
    \255\255\255\255\037\000\037\000\255\255\255\255\037\000\034\000\
    \084\000\034\000\255\255\035\000\035\000\035\000\029\000\027\000\
    \255\255\255\255\036\000\086\000\255\255\255\255\035\000\036\000\
    \035\000\037\000\255\255\255\255\034\000\034\000\255\255\034\000\
    \047\000\255\255\048\000\037\000\255\255\036\000\030\000\255\255\
    \037\000\255\255\049\000\035\000\035\000\255\255\035\000\087\000\
    \255\255\255\255\255\255\082\000\255\255\031\000\037\000\038\000\
    \255\255\255\255\038\000\039\000\255\255\255\255\039\000\255\255\
    \067\000\081\000\255\255\036\000\036\000\036\000\255\255\255\255\
    \255\255\255\255\080\000\255\255\032\000\255\255\036\000\099\000\
    \036\000\255\255\099\000\255\255\037\000\037\000\037\000\038\000\
    \051\000\094\000\094\000\039\000\038\000\094\000\255\255\037\000\
    \039\000\037\000\255\255\036\000\036\000\101\000\036\000\083\000\
    \101\000\255\255\038\000\255\255\255\255\255\255\039\000\093\000\
    \094\000\255\255\255\255\102\000\037\000\037\000\102\000\037\000\
    \100\000\100\000\255\255\033\000\100\000\046\000\046\000\255\255\
    \255\255\046\000\053\000\255\255\106\000\053\000\255\255\106\000\
    \038\000\038\000\038\000\084\000\039\000\039\000\039\000\100\000\
    \255\255\255\255\255\255\038\000\046\000\038\000\086\000\039\000\
    \255\255\039\000\255\255\255\255\255\255\255\255\046\000\255\255\
    \255\255\034\000\053\000\046\000\255\255\255\255\255\255\053\000\
    \038\000\038\000\255\255\038\000\039\000\039\000\255\255\039\000\
    \035\000\046\000\087\000\255\255\255\255\053\000\054\000\082\000\
    \255\255\054\000\255\255\255\255\255\255\255\255\255\255\097\000\
    \255\255\255\255\099\000\255\255\103\000\055\000\255\255\255\255\
    \055\000\255\255\255\255\255\255\255\255\094\000\255\255\046\000\
    \046\000\046\000\255\255\053\000\053\000\053\000\054\000\255\255\
    \101\000\255\255\046\000\054\000\046\000\255\255\053\000\255\255\
    \053\000\255\255\255\255\255\255\107\000\055\000\102\000\255\255\
    \036\000\054\000\055\000\056\000\100\000\255\255\056\000\046\000\
    \046\000\053\000\046\000\053\000\053\000\255\255\053\000\106\000\
    \055\000\037\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\054\000\
    \054\000\054\000\255\255\056\000\255\255\255\255\255\255\255\255\
    \056\000\255\255\054\000\255\255\054\000\255\255\055\000\055\000\
    \055\000\255\255\255\255\054\000\255\255\255\255\056\000\057\000\
    \255\255\055\000\057\000\055\000\255\255\255\255\255\255\054\000\
    \054\000\255\255\054\000\255\255\255\255\038\000\255\255\058\000\
    \055\000\039\000\058\000\255\255\255\255\255\255\055\000\055\000\
    \255\255\055\000\255\255\255\255\056\000\056\000\056\000\057\000\
    \255\255\255\255\255\255\255\255\057\000\099\000\255\255\056\000\
    \255\255\056\000\255\255\255\255\255\255\255\255\255\255\058\000\
    \094\000\255\255\057\000\255\255\058\000\056\000\255\255\255\255\
    \255\255\255\255\255\255\101\000\056\000\056\000\255\255\056\000\
    \255\255\255\255\058\000\059\000\255\255\255\255\059\000\255\255\
    \255\255\102\000\255\255\255\255\255\255\255\255\255\255\100\000\
    \057\000\057\000\057\000\255\255\046\000\255\255\255\255\255\255\
    \053\000\255\255\106\000\057\000\255\255\057\000\255\255\255\255\
    \058\000\058\000\058\000\059\000\255\255\255\255\255\255\058\000\
    \059\000\060\000\057\000\058\000\060\000\058\000\255\255\255\255\
    \057\000\057\000\255\255\057\000\255\255\255\255\059\000\255\255\
    \061\000\255\255\255\255\061\000\255\255\255\255\255\255\060\000\
    \058\000\058\000\255\255\058\000\255\255\255\255\255\255\255\255\
    \255\255\060\000\255\255\255\255\054\000\255\255\060\000\255\255\
    \255\255\255\255\255\255\255\255\059\000\059\000\059\000\255\255\
    \061\000\255\255\255\255\055\000\060\000\061\000\059\000\059\000\
    \255\255\059\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\061\000\061\000\063\000\255\255\255\255\
    \063\000\255\255\255\255\255\255\059\000\059\000\255\255\059\000\
    \255\255\255\255\060\000\060\000\060\000\255\255\255\255\064\000\
    \255\255\056\000\064\000\255\255\255\255\060\000\255\255\060\000\
    \255\255\061\000\061\000\061\000\255\255\063\000\255\255\255\255\
    \255\255\255\255\063\000\255\255\061\000\255\255\061\000\255\255\
    \255\255\255\255\060\000\060\000\255\255\060\000\255\255\064\000\
    \063\000\063\000\255\255\065\000\064\000\255\255\065\000\255\255\
    \255\255\061\000\061\000\255\255\061\000\255\255\255\255\255\255\
    \255\255\255\255\064\000\064\000\066\000\057\000\255\255\066\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\063\000\063\000\
    \063\000\255\255\255\255\065\000\255\255\058\000\255\255\255\255\
    \065\000\063\000\255\255\063\000\255\255\255\255\255\255\255\255\
    \064\000\064\000\064\000\255\255\066\000\255\255\065\000\065\000\
    \255\255\066\000\255\255\064\000\255\255\064\000\063\000\063\000\
    \255\255\063\000\255\255\070\000\255\255\255\255\070\000\066\000\
    \255\255\071\000\255\255\255\255\071\000\255\255\255\255\255\255\
    \064\000\064\000\255\255\064\000\065\000\065\000\065\000\255\255\
    \255\255\059\000\255\255\255\255\255\255\255\255\255\255\065\000\
    \255\255\065\000\255\255\070\000\255\255\066\000\066\000\066\000\
    \070\000\071\000\255\255\255\255\255\255\255\255\071\000\255\255\
    \066\000\255\255\066\000\255\255\065\000\065\000\070\000\065\000\
    \072\000\255\255\255\255\072\000\071\000\255\255\255\255\060\000\
    \255\255\255\255\255\255\255\255\255\255\066\000\066\000\255\255\
    \066\000\255\255\255\255\255\255\255\255\255\255\061\000\255\255\
    \255\255\255\255\255\255\255\255\070\000\070\000\070\000\255\255\
    \072\000\255\255\071\000\071\000\071\000\072\000\255\255\070\000\
    \255\255\070\000\255\255\255\255\255\255\071\000\255\255\071\000\
    \072\000\255\255\255\255\072\000\255\255\070\000\255\255\071\000\
    \255\255\255\255\255\255\255\255\070\000\070\000\255\255\070\000\
    \255\255\255\255\071\000\071\000\255\255\071\000\255\255\255\255\
    \073\000\073\000\255\255\063\000\073\000\255\255\255\255\255\255\
    \255\255\072\000\072\000\072\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\072\000\064\000\072\000\073\000\
    \073\000\073\000\255\255\255\255\255\255\255\255\073\000\255\255\
    \255\255\073\000\255\255\073\000\255\255\073\000\073\000\255\255\
    \255\255\072\000\072\000\255\255\072\000\255\255\255\255\255\255\
    \255\255\073\000\073\000\255\255\073\000\255\255\073\000\074\000\
    \074\000\065\000\255\255\074\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\066\000\255\255\255\255\255\255\074\000\074\000\
    \074\000\255\255\073\000\073\000\073\000\074\000\255\255\255\255\
    \255\255\255\255\074\000\255\255\074\000\073\000\255\255\073\000\
    \255\255\255\255\255\255\075\000\075\000\255\255\255\255\075\000\
    \074\000\074\000\255\255\255\255\255\255\074\000\255\255\255\255\
    \255\255\255\255\073\000\073\000\255\255\073\000\255\255\255\255\
    \255\255\070\000\075\000\075\000\075\000\255\255\255\255\071\000\
    \255\255\075\000\255\255\255\255\075\000\255\255\075\000\255\255\
    \075\000\075\000\255\255\255\255\255\255\076\000\076\000\255\255\
    \255\255\076\000\255\255\077\000\075\000\075\000\077\000\075\000\
    \255\255\075\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\076\000\076\000\076\000\255\255\
    \255\255\255\255\078\000\076\000\255\255\078\000\072\000\255\255\
    \076\000\255\255\076\000\077\000\255\255\075\000\075\000\075\000\
    \077\000\255\255\255\255\255\255\255\255\255\255\076\000\076\000\
    \075\000\255\255\075\000\076\000\255\255\255\255\077\000\255\255\
    \255\255\255\255\078\000\255\255\255\255\255\255\255\255\078\000\
    \255\255\255\255\255\255\255\255\255\255\075\000\075\000\255\255\
    \075\000\255\255\078\000\255\255\255\255\078\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\077\000\077\000\077\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\077\000\
    \255\255\077\000\255\255\255\255\255\255\255\255\255\255\073\000\
    \255\255\077\000\255\255\078\000\078\000\078\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\077\000\077\000\078\000\077\000\
    \078\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\078\000\078\000\255\255\078\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\074\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\075\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\076\000\255\255\255\255\255\255\
    \255\255\077\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \078\000";
  Lexing.lex_base_code = 
   "";
  Lexing.lex_backtrk_code = 
   "";
  Lexing.lex_default_code = 
   "";
  Lexing.lex_trans_code = 
   "";
  Lexing.lex_check_code = 
   "";
  Lexing.lex_code = 
   "";
}

let rec parse_bol c lexbuf =
    __ocaml_lex_parse_bol_rec c lexbuf 0
and __ocaml_lex_parse_bol_rec c lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 279 "wikicreole.mll"
               (
      end_paragraph c 0;
      parse_bol c lexbuf
    )
# 693 "wikicreole.ml"

  | 1 ->
# 283 "wikicreole.mll"
                                                                     (
      end_paragraph c 0;
      assert (c.stack = Paragraph);
      c.stack <- Heading (count '=' (Lexing.lexeme lexbuf));
      c.heading <- true;
      parse_rem c lexbuf
    )
# 704 "wikicreole.ml"

  | 2 ->
# 290 "wikicreole.mll"
                        (
      let lev = count '*' (Lexing.lexeme lexbuf) in
      if not (start_list_item c Unordered lev) then begin
        let s = Lexing.lexeme lexbuf in
        let l = String.length s - lev in
        if l > 0 then push_string c (String.sub s 0 l);
        for i = 1 to lev / 2 do
          style_change c Bold
        done;
        if lev land 1 = 1 then push_string c "*"
      end;
      parse_rem c lexbuf
    )
# 721 "wikicreole.ml"

  | 3 ->
# 303 "wikicreole.mll"
                        (
      let lev = count '#' (Lexing.lexeme lexbuf) in
      if not (start_list_item c Ordered lev) then
        push_chars c lexbuf;
      parse_rem c lexbuf
    )
# 731 "wikicreole.ml"

  | 4 ->
# 309 "wikicreole.mll"
                                                          (
      end_paragraph c 0;
      c.flow <- c.build.hr_elem () :: c.flow;
      parse_bol c lexbuf
    )
# 740 "wikicreole.ml"

  | 5 ->
# 314 "wikicreole.mll"
                                           (
      end_paragraph c 0;
      parse_nowiki c lexbuf
    )
# 748 "wikicreole.ml"

  | 6 ->
# 318 "wikicreole.mll"
                      (
      start_table_row c false;
      parse_rem c lexbuf
    )
# 756 "wikicreole.ml"

  | 7 ->
# 322 "wikicreole.mll"
                       (
      start_table_row c true;
      parse_rem c lexbuf
    )
# 764 "wikicreole.ml"

  | 8 ->
# 326 "wikicreole.mll"
       (
      parse_rem c lexbuf
    )
# 771 "wikicreole.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; 
      __ocaml_lex_parse_bol_rec c lexbuf __ocaml_lex_state

and parse_rem c lexbuf =
    __ocaml_lex_parse_rem_rec c lexbuf 25
and __ocaml_lex_parse_rem_rec c lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 332 "wikicreole.mll"
               (
      (* Headings are single lines *)
      if c.heading then
        end_paragraph c 0
      else
        push_chars c lexbuf;
      parse_bol c lexbuf
    )
# 790 "wikicreole.ml"

  | 1 ->
# 340 "wikicreole.mll"
         (
      style_change c Bold;
      parse_rem c lexbuf
    )
# 798 "wikicreole.ml"

  | 2 ->
# 344 "wikicreole.mll"
         (
      style_change c Italic;
      parse_rem c lexbuf
    )
# 806 "wikicreole.ml"

  | 3 ->
# 348 "wikicreole.mll"
                                           (
      if c.heading then
        end_paragraph c 0
      else
        push_chars c lexbuf;
      parse_bol c lexbuf
    )
# 817 "wikicreole.ml"

  | 4 ->
# 355 "wikicreole.mll"
                                                       (
      if c.link then
        push_chars c lexbuf
      else
        let s = Lexing.lexeme lexbuf in
        let addr = String.sub s 2 (String.length s - 4) in
        c.inline_mix <-
         c.build.a_elem addr [c.build.chars addr] :: c.inline_mix;
      parse_rem c lexbuf
  )
# 831 "wikicreole.ml"

  | 5 ->
# 365 "wikicreole.mll"
                                                           (
      if c.link then
        push_chars c lexbuf
      else
        let s = Lexing.lexeme lexbuf in
        let addr = String.sub s 10 (String.length s - 12) in
        c.inline_mix <-
         c.build.youtube_elem addr [c.build.chars addr] :: c.inline_mix;
      parse_rem c lexbuf
  )
# 845 "wikicreole.ml"

  | 6 ->
# 375 "wikicreole.mll"
                                                      (
      if c.link then
        push_chars c lexbuf
      else begin
        let s = Lexing.lexeme lexbuf in
        let addr = String.sub s 2 (String.length s - 3) in
        c.stack <- Link (addr, c.stack);
        c.link <- true
      end;
      parse_rem c lexbuf
  )
# 860 "wikicreole.ml"

  | 7 ->
# 386 "wikicreole.mll"
         (
      begin match c.stack with
        Link (addr, stack) ->
          pop_link c addr stack
      | _ ->
          push_chars c lexbuf
      end;
      parse_rem c lexbuf
    )
# 873 "wikicreole.ml"

  | 8 ->
# 396 "wikicreole.mll"
                                                 (
      if c.link then
        push_chars c lexbuf
      else
        let addr = Lexing.lexeme lexbuf in
        c.inline_mix <-
          c.build.a_elem addr [c.build.chars addr] :: c.inline_mix;
      parse_rem c lexbuf
  )
# 886 "wikicreole.ml"

  | 9 ->
# 405 "wikicreole.mll"
           (
      push c (c.build.br_elem ());
      parse_rem c lexbuf
    )
# 894 "wikicreole.ml"

  | 10 ->
# 410 "wikicreole.mll"
                                               (
      let s = Lexing.lexeme lexbuf in
      let i = String.index s '|' in
      let url = String.sub s 2 (i - 2) in
      let alt = String.sub s (i + 1) (String.length s - i - 3) in
      push c (c.build.img_elem url alt);
      parse_rem c lexbuf
    )
# 906 "wikicreole.ml"

  | 11 ->
# 418 "wikicreole.mll"
                                                            (
      let s = Lexing.lexeme lexbuf in
      let txt = String.sub s 3 (String.length s - 6) in
      push c (c.build.tt_elem [c.build.inline (c.build.chars txt)]);
      parse_rem c lexbuf
    )
# 916 "wikicreole.ml"

  | 12 ->
# 424 "wikicreole.mll"
                                       (
      let s = Lexing.lexeme lexbuf in
      (* It amounts to the same to quote a UTF-8 char or its first byte *)
      push_string c (String.sub s 1 1);
      parse_rem c lexbuf
    )
# 926 "wikicreole.ml"

  | 13 ->
# 430 "wikicreole.mll"
                                        (
      if not (close_row c) then
        push_chars c lexbuf;
      parse_bol c lexbuf
    )
# 935 "wikicreole.ml"

  | 14 ->
# 435 "wikicreole.mll"
        (
      if close_entry c then
        c.stack <- Entry (false, c.stack)
      else
        push_chars c lexbuf;
      parse_rem c lexbuf
    )
# 946 "wikicreole.ml"

  | 15 ->
# 442 "wikicreole.mll"
         (
      if close_entry c then
        c.stack <- Entry (true, c.stack)
      else
        push_chars c lexbuf;
      parse_rem c lexbuf
    )
# 957 "wikicreole.ml"

  | 16 ->
# 449 "wikicreole.mll"
                                  (
      push_chars c lexbuf;
      parse_rem c lexbuf
    )
# 965 "wikicreole.ml"

  | 17 ->
# 453 "wikicreole.mll"
        (
      end_paragraph c 0
    )
# 972 "wikicreole.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; 
      __ocaml_lex_parse_rem_rec c lexbuf __ocaml_lex_state

and parse_nowiki c lexbuf =
    __ocaml_lex_parse_nowiki_rec c lexbuf 94
and __ocaml_lex_parse_nowiki_rec c lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 459 "wikicreole.mll"
                                           (
      let s = Lexing.lexeme lexbuf in
      c.pre_content <- String.sub s 1 (String.length s - 1) :: c.pre_content;
      parse_nowiki c lexbuf
    )
# 988 "wikicreole.ml"

  | 1 ->
# 464 "wikicreole.mll"
                                     (
      c.flow <- c.build.pre_elem (List.rev c.pre_content) :: c.flow;
      c.pre_content <- [];
      parse_bol c lexbuf
    )
# 997 "wikicreole.ml"

  | 2 ->
# 469 "wikicreole.mll"
                                        (
      c.pre_content <- Lexing.lexeme lexbuf :: c.pre_content;
      parse_nowiki c lexbuf
    )
# 1005 "wikicreole.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; 
      __ocaml_lex_parse_nowiki_rec c lexbuf __ocaml_lex_state

;;

# 474 "wikicreole.mll"
 

let context b =
  { build = b; italic = false; bold = false;
    heading = false; link = false; list_level = 0;
    inline_mix = []; link_content = []; pre_content = []; list = []; flow = [];
    stack = Paragraph }

let from_lexbuf b lexbuf =
  let c = context b in
  parse_bol c lexbuf;
  List.rev c.flow

let from_channel b ch = from_lexbuf b (Lexing.from_channel ch)

let from_string b s = from_lexbuf b (Lexing.from_string s)


# 1031 "wikicreole.ml"
